import os
import sys
import glob
import ablog
import shutil
import argparse


def find_confdir():
    """Return path to current directory or its parent that contains conf.py"""

    from os.path import isfile, join, abspath
    confdir = os.getcwd()

    parent = lambda d: abspath(join(d, '..'))

    while not isfile(join(confdir, 'conf.py')) and confdir != parent(confdir):
        confdir = parent(confdir)

    conf = join(confdir, 'conf.py')

    if isfile(conf) and 'ablog' in open(conf).read():
        return confdir
    else:
        sys.exit("Current directory and its parents doesn't "
            "contain configuration file (conf.py).")


def read_conf(confdir):
    """Return conf.py file as a module."""

    sys.path.insert(0, confdir)
    conf = __import__('conf')
    sys.path.pop(0)
    return conf


parser = argparse.ArgumentParser(
    description="ABlog for blogging with Sphinx",
    epilog="See 'ablog <command> -h' for more information on a specific "
           "command.")

parser.add_argument('-v', '--version',
    help="print ABlog version and exit",
    action='version', version=ablog.__version__)


commands = ablog_commands = parser.add_subparsers(title='commands')

def cmd(func=None, **kwargs):

    if func is None:
        def cmd_inner(func):
            return cmd(func, **kwargs)
        return cmd_inner
    else:
        command = commands.add_parser(**kwargs)
        command.set_defaults(func=func)
        command.set_defaults(subparser=command)
        func.command = command
        return func

def arg(*args, **kwargs):
    if args and callable(args[0]):
        func = args[0]
        args = args[1:]
    else:
        func = None
    if func is None:
        def arg_inner(func):
            return arg(func, *args, **kwargs)
        return arg_inner
    else:
        func.command.add_argument(*args, **kwargs)
        return func

def arg_website(func):

    arg(func, '-w', dest='website', type=str,
        help="path for website, default is _website when `ablog_website` "
            "is not set in conf.py")
    return func

def arg_doctrees(func):

    arg(func, '-d', dest='doctrees', type=str, default='.doctrees',
        help="path for the cached environment and doctree files, "
            "default .doctrees when `ablog_doctrees` is not set in conf.py")
    return func


from .start import ablog_start
cmd(ablog_start, name='start', help='start a new blog project',
    description="Start a new blog project by answering a few questions. "
    "You will end up with a configuration file and sample pages.")


@arg('-P', dest='runpdb',
    action='store_true', default=False,
    help="run pdb on exception")
@arg('-T', dest='traceback',
    action='store_true', default=False,
    help="show full traceback on exception")
@arg_doctrees
@arg_website
@arg('-s', dest='sourcedir', type=str,
    help="root path for source files, "
        "default is path to the folder that contains conf.py")
@arg('-b', dest='builder', type=str,
    help="builder to use, default `ablog_builder` or dirhtml")
@arg('-a', dest='allfiles',     action='store_true', default=False,
    help="write all files; default is to only write new and changed files")
@cmd(name='build', help='build your blog project',
    description="Path options can be set in conf.py. "
    "Default values of paths are relative to conf.py.")
def ablog_build(builder=None, sourcedir=None, website=None, doctrees=None,
    traceback=False, runpdb=False, allfiles=False, **kwargs):

    confdir = find_confdir()
    conf = read_conf(confdir)
    website = (website or
        os.path.join(confdir, getattr(conf, 'ablog_builddir', '_website')))
    doctrees = (doctrees or
        os.path.join(confdir, getattr(conf, 'ablog_doctrees', '.doctrees')))
    sourcedir = (sourcedir or confdir)
    argv = sys.argv[:1]
    argv.extend(['-b', builder or getattr(conf, 'ablog_builder', 'dirhtml')])
    argv.extend(['-d', doctrees])
    if traceback:
        argv.extend(['-T'])
    if runpdb:
        argv.extend(['-P'])
    if allfiles:
        argv.extend(['-a'])
    argv.extend([sourcedir, website])

    from sphinx import main
    main(argv)


@arg('-D', dest='deep', action='store_true', default=False,
    help="deep clean, remove cached environment and doctree files")
@arg_doctrees
@arg_website
@cmd(name='clean', help='clean your blog build files',
    description="Path options can be set in conf.py. "
    "Default values of paths are relative to conf.py.")
def ablog_clean(website=None, doctrees=None, deep=False, **kwargs):

    confdir = find_confdir()
    conf = read_conf(confdir)

    website = (website or
        os.path.join(confdir, getattr(conf, 'ablog_builddir', '_website')))

    doctrees = (doctrees or
        os.path.join(confdir, getattr(conf, 'ablog_doctrees', '.doctrees')))

    nothing = True
    if glob.glob(os.path.join(website, '*')):
        shutil.rmtree(website)
        print('Removed {}.'.format(os.path.relpath(website)))
        nothing = False

    if deep and glob.glob(os.path.join(doctrees, '*')):
        shutil.rmtree(doctrees)
        print('Removed {}.'.format(os.path.relpath(doctrees)))
        nothing = False

    if nothing:
        print('Nothing to clean.')


@arg('--patterns', dest='rebuild', action='store_false', default='*.rst;*.txt',
    help="patterns for triggering rebuilds")
@arg('-r', dest='rebuild', action='store_true', default=False,
    help="rebuild when a file matching patterns change or get added")
@arg('-n', dest='view', action='store_false', default=True,
    help="do not open website in a new browser tab")
@arg('-p', dest='port', type=int, default=8000,
    help='port number for HTTP server; default is 8000')
@arg_website
@cmd(name='serve', help='serve and view your project',
    description="Serve options can be set in conf.py. "
    "Default values of paths are relative to conf.py.")
def ablog_serve(website=None, port=8000, view=True, rebuild=False,
    patterns='*.rst;*.txt', **kwargs):

    confdir = find_confdir()
    conf = read_conf(confdir)

    try:
        import SimpleHTTPServer as server
    except ImportError:
        from http import server
        import socketserver
    else:
        import SocketServer as socketserver

    import webbrowser

    Handler = server.SimpleHTTPRequestHandler
    httpd = socketserver.TCPServer(("", port), Handler)

    ip, port = httpd.socket.getsockname()
    print("Serving HTTP on {}:{}.".format(ip, port))
    print("Quit the server with Control-C.")

    website = (website or
        os.path.join(confdir, getattr(conf, 'ablog_builddir', '_website')))

    os.chdir(website)

    if rebuild:

        #from watchdog.watchmedo import observe_with
        from watchdog.observers import Observer
        from watchdog.tricks import ShellCommandTrick
        patterns = patterns.split(';')
        ignore_patterns = [os.path.join(website, '*')]
        handler = ShellCommandTrick(shell_command='ablog build',
                                    patterns=patterns,
                                    ignore_patterns=ignore_patterns,
                                    ignore_directories=False,
                                    wait_for_process=True,
                                    drop_during_process=False)

        observer = Observer(timeout=1)
        observer.schedule(handler, confdir, recursive=True)
        observer.start()
        try:
            if view:
                (webbrowser.open_new_tab('http://127.0.0.1:{}'.format(port)) and
                 httpd.serve_forever())
            else:
                httpd.serve_forever()
        except KeyboardInterrupt:
            observer.stop()
        observer.join()


    else:
        if view:
            (webbrowser.open_new_tab('http://127.0.0.1:{}'.format(port)) and
             httpd.serve_forever())
        else:
            httpd.serve_forever()


@arg('-t', dest='title', type=str,
    help='post title; default is formed from filename')
@arg(dest='filename', type=str,
    help='filename, e.g. my-nth-post (.rst appended)')
@cmd(name='post', help='create a blank post',)
def ablog_post(filename, title=None, **kwargs):

    POST_TEMPLATE =u'''
%(title)s
%(equal)s

.. post:: %(date)s
   :tags:
   :category:

'''
    from datetime import date
    from os import path

    #Generate basic post params.
    today = date.today()
    if not filename.lower().endswith('.rst'):
        filename += '.rst'

    today = today.strftime("%b %d, %Y")
    if not title:
        title = filename[:-4].replace('-', ' ').title()

    pars = {'date': today,
            'title': title,
            'equal': '=' * len(title)
            }

    if path.isfile(filename):
        pass
        # read the file, and add post directive
        # and save it
    else:
        with open(filename, 'w') as out:
            post_text = POST_TEMPLATE % pars
            out.write(post_text)

        print('Blog post created: %s' % filename)


@arg('--github-token', dest='github_token', type=str,
    help="environment variable name storing GitHub access token")
@arg('--push-quietly', dest='push_quietly', action='store_true', default=False,
    help="be more quiet when pushing changes")
@arg('-f', dest='push_force', action='store_true', default=False,
    help="owerwrite last commit, i.e. `commit --amend; push -f`")
@arg('-m', dest='message', type=str, help="commit message")
@arg('-g', dest='github_pages', type=str,
    help="GitHub username for deploying to GitHub pages")
@arg('-p', dest='repodir', type=str,
    help="path to the location of repository to be deployed, e.g. "
         "`../username.github.io`, default is folder containing `conf.py`")
@arg_website
@cmd(name='deploy', help='deploy your website build files',
    description="Path options can be set in conf.py. "
    "Default values of paths are relative to conf.py.")
def ablog_deploy(website, message=None, github_pages=None,
    push_quietly=False, push_force=False, github_token=None, repodir=None,
    **kwargs):

    confdir = find_confdir()
    conf = read_conf(confdir)

    github_pages = (github_pages or getattr(conf, 'github_pages') or None)

    website = (website or
        os.path.join(confdir, getattr(conf, 'ablog_builddir', '_website')))

    tomove = glob.glob(os.path.join(website, '*'))
    if not tomove:
        print('Nothing to deploy, build first.')
        return

    try:
        from invoke import run
    except ImportError:
        raise ImportError("invoke is required by deploy command, "
            "run `pip install invoke`")


    if github_pages:

        if repodir is None:
            repodir = os.path.join(confdir, "{0}.github.io".format(github_pages))
        if os.path.isdir(repodir):
            os.chdir(repodir)
            run("git pull", echo=True)
        else:
            run("git clone https://github.com/{0}/{0}.github.io.git {1}"
                .format(github_pages, repodir), echo=True)

        git_add = []
        for tm in tomove:
            for root, dirnames, filenames in os.walk(website):
                for filename in filenames:
                    fn = os.path.join(root, filename)
                    fnnew = fn.replace(website, repodir)
                    try:
                        os.renames(fn, fnnew)
                    except OSError:
                        if os.path.isdir(fnnew):
                            shutil.rmtree(fnnew)
                        else:
                            os.remove(fnnew)
                        os.renames(fn, fnnew)

                    git_add.append(fnnew)
        print('Moved {} files to {}.github.io'
            .format(len(git_add), github_pages))

        os.chdir(repodir)

        run("git add -f " + " ".join(['"{}"'.format(os.path.relpath(p))
                                      for p in git_add]), echo=True)
        if not os.path.isfile('.nojekyll'):
            open('.nojekyll', 'w')
            run("git add -f .nojekyll")

        commit = 'git commit -m "{}"'.format(message or 'Updates.')
        if push_force:
            commit += ' --amend'
        run(commit, echo=True)

        if github_token:
            with open(os.path.join(repodir, '.git/credentials'), 'w') as out:
                out.write('https://{}:@github.com'
                    .format(os.environ[github_token]))
            run('git config credential.helper "store --file=.git/credentials"')
        push = 'git push'
        if push_quietly:
            push += ' -q'
        if push_force:
            push += ' -f'
        push += ' origin master'
        run(push, echo=True)

    else:
        print('No place to deploy.')


def ablog_main():

    if len(sys.argv) == 1:
        parser.print_help()
    else:
        namespace = parser.parse_args()
        namespace.func(**namespace.__dict__)


if __name__ == '__main__':
    ablog_main()



# -*- coding: utf-8 -*-
"""ABlog for Sphinx"""
import os

from .blog import Blog, CONFIG
from .post import (PostDirective, PostListDirective, UpdateDirective,
                   UpdateNode, process_posts, process_postlist, purge_posts,
                   generate_archive_pages, generate_atom_feeds,
                   missing_reference)

__version__ = '0.8.0'


def anchor(post):
    """Return anchor string for posts that arepage sections."""

    if post.section:
        return '#' + post.section
    else:
        return ''

def builder_support(builder):
    """Return True when builder is supported. Supported builders output in
    html format, but exclude `PickleHTMLBuilder` and `JSONHTMLBuilder`,
    which run into issues when serializing blog objects."""

    if hasattr(builder, 'builder'):
        builder = builder.builder

    not_supported = {'json', 'pickle'}
    return builder.format == 'html' and not builder.name in not_supported


def html_page_context(app, pagename, templatename, context, doctree):

    if builder_support(app):
        context['ablog'] = Blog(app)
        context['anchor'] = anchor


def setup(app):
    """Setup ABlog extension."""

    for args in CONFIG:
        app.add_config_value(*args)

    app.add_directive('post', PostDirective)
    app.add_directive('postlist', PostListDirective)

    app.connect('doctree-read', process_posts)

    app.connect('env-purge-doc', purge_posts)
    app.connect('doctree-resolved', process_postlist)
    app.connect('missing-reference', missing_reference)
    app.connect('html-collect-pages', generate_archive_pages)
    app.connect('html-collect-pages', generate_atom_feeds)
    app.connect('html-page-context', html_page_context)

    app.add_directive('update', UpdateDirective)
    app.add_node(UpdateNode,
                 html=(lambda s, n: s.visit_admonition(n),
                       lambda s, n: s.depart_admonition(n)),
                 latex=(lambda s, n: s.visit_admonition(n),
                       lambda s, n: s.depart_admonition(n)),
                 )

    pkgdir = os.path.abspath(os.path.dirname(__file__))
    locale_dir = os.path.join(pkgdir, 'locale')
    app.config.locale_dirs.append(locale_dir)


def get_html_templates_path():
    """Return path to ABlog templates folder."""

    pkgdir = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(pkgdir, 'templates')



# -*- coding: utf-8 -*-
"""Classes for handling posts and archives."""

import os
import re
import sys

import datetime as dtmod
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin
from datetime import datetime
from unicodedata import normalize

from docutils import nodes
from docutils.io import StringOutput

from sphinx import addnodes
from sphinx.util.osutil import relative_uri
from sphinx.environment import dummy_reporter

if sys.version_info >= (3, 0):
    text_type = str
    re_flag = 0
else:
    text_type = unicode
    re_flag = re.UNICODE

def slugify(string):
    """Slugify *s*."""

    string = text_type(string)
    string = normalize('NFKD', string)

    string = re.sub(r'[^\w\s-]', '', string, flags=re_flag).strip().lower()
    return re.sub(r'[-\s]+', '-', string, flags=re_flag)

def os_path_join(path, *paths):

    return os.path.join(path, *paths).replace(os.path.sep, '/')


DEBUG = True
CONFIG = [
    # name, default, rebuild
    ('blog_path', 'blog', True),
    ('blog_title', 'Blog', True),
    ('blog_baseurl', None, True),
    ('blog_archive_titles', None, False),

    ('blog_feed_archives', False, True),
    ('blog_feed_fulltext', False, True),
    ('blog_feed_subtitle', None, True),
    ('blog_feed_titles', None, False),
    ('blog_feed_length', None, None),

    ('blog_authors', {}, True),
    ('blog_default_author', None, True),
    ('blog_locations', {}, True),
    ('blog_default_location', None, True),
    ('blog_languages', {}, True),
    ('blog_default_language', None, True),

    ('fontawesome_link_cdn', False, True),
    ('fontawesome_included', False, True),
    ('fontawesome_css_file', False, True),

    ('post_date_format', '%b %d, %Y', True),
    ('post_date_format_short', '%b %d', True),
    ('post_auto_image', 0, True),
    ('post_auto_excerpt', 1, True),
    ('post_redirect_refresh', 5, True),
    ('post_always_section', False, True),

    ('disqus_shortname', None, True),
    ('disqus_drafts', False, True),
    ('disqus_pages', False, True),
]


TOMORROW = datetime.today() + dtmod.timedelta(1)
FUTURE = datetime(9999, 12, 31)


def revise_pending_xrefs(doctree, docname):

    for node in doctree.traverse(addnodes.pending_xref):
        node['refdoc'] = docname


class Blog(object):

    """Handle blog operations."""

    # using a shared state
    _dict = {}

    def __init__(self, app):

        self.__dict__ = self._dict
        if not self._dict:
            self._init(app)

    def _init(self, app):
        """Instantiate Blog."""

        self.app = app
        self.config = {}

        self.references = refs = {}

        # get configuration from Sphinx app
        for opt in CONFIG:
            self.config[opt[0]] = getattr(app.config, opt[0])

        opt = self.config['blog_default_author']
        if opt is not None and not isinstance(opt, list):
            self.config['blog_default_author'] = [opt]

        opt = self.config['blog_default_location']
        if opt is not None and not isinstance(opt, list):
            self.config['blog_default_location'] = [opt]

        opt = self.config['blog_default_language']
        if opt is not None and not isinstance(opt, list):
            self.config['blog_default_language'] = [opt]

        # blog catalog contains all posts
        self.blog = Catalog(self, 'blog', 'blog', None)

        # contains post collections by year
        self.archive = Catalog(self, 'archive', 'archive', None, reverse=True)
        self.archive.docname += '/archive'
        refs['blog-archives'] = (self.archive.docname, 'Archives')

        self.catalogs = cat = {}  # catalogs of user set labels
        self.tags = cat['tags'] = Catalog(self, 'tags', 'tag', 'tag')
        refs['blog-tags'] = (self.tags.docname, 'Tags')

        self.author = cat['author'] = Catalog(self, 'author',
            'author', 'author')
        refs['blog-authors'] = (self.author.docname, 'Authors')

        self.location = cat['location'] = Catalog(self, 'location',
            'location',  'location')
        refs['blog-locations'] = (self.location.docname, 'Locations')

        self.language = cat['language'] = Catalog(self, 'language',
            'language',  'language')
        refs['blog-languages'] = (self.language.docname, 'Languages')

        self.category = cat['category'] = Catalog(self, 'category',
            'category', 'category')
        refs['blog-categories'] = (self.category.docname, 'Categories')

        for catname in ['author', 'location', 'language']:
            catalog = self.catalogs[catname]
            items = self.config['blog_' + catname + 's'].items()
            for label, (name, link) in items:
                catalog[label] = Collection(catalog, label, name, link)

        self.posts = self.blog['post'] = Collection(self.blog, 'post',
            'Posts', path=self.blog_path)
        self.drafts = self.blog['draft'] = Collection(self.blog, 'draft',
            'Drafts', path=os_path_join(self.blog_path, 'drafts'))

        # add references to posts and drafts
        # e.g. :ref:`blog-posts`
        refs['blog-posts'] = (os_path_join(self.config['blog_path'], 'index'), 'Posts')
        refs['blog-drafts'] = (os_path_join(self.config['blog_path'], 'drafts', 'index'), 'Drafts')
        refs['blog-feed'] = (os_path_join(self.config['blog_path'], 'atom.xml'), self.blog_title + ' Feed')

        # set some internal configuration options
        self.config['fontawesome'] = (self.config['fontawesome_included'] or
                                      self.config['fontawesome_link_cdn'] or
                                      self.config['fontawesome_css_file'])

    def __getattr__(self, name):

        try:
            attr = self.config[name]
        except KeyError:
            raise AttributeError('ABlog has no configuration option {}'
                                 .format(repr(name)))
        return attr

    def __getitem__(self, key):

        return self.posts[key] or self.drafts[key]

    def __contains__(self, item):

        return item in self.posts or item in self.drafts

    def __len__(self):

        return len(self.posts)

    def __nonzero__(self):

        return len(self) > 0

    @property
    def feed_path(self):
        """RSS feed page name."""

        return os_path_join(self.blog_path, 'rss.xml')

    def register(self, docname, info):
        """Register post *docname*."""

        post = Post(self, docname, info)
        if post.date and post.date < TOMORROW:
            self.posts.add(post)
        else:
            self.drafts.add(post)
        for catalog in self.catalogs.values():
            catalog.add(post)

    def recent(self, num, docname=None, **labels):
        """Yield *num* recent posts, excluding the one with `docname`."""

        if num is None:
            num = len(self)
        for i, post in enumerate(self.posts):
            if post.docname == docname:
                num += 1
                continue
            if i == num:
                return
            yield post

    def link_posts(self):
        """Link posts after sorting them post by published date."""

        if not getattr(self, '_posts_sorted', False):
            posts = [post for post in self.posts if post.order == 1]
            posts.sort()
            posts[0].prev = posts[-1].next = None
            for i in range(1, len(posts)):
                post = posts[i]
                posts[i - 1].next = post
                post.prev = posts[i - 1]
            self._posts_sorted = True

    def page_id(self, pagename):
        """Page identifier for Disqus."""

        if self.config['blog_baseurl']:
            if pagename.endswith('index'):
                pagename = pagename[:-5]
            pagename = pagename.strip('/')
            return '/' + pagename + ('/' if pagename else '')

    def page_url(self, pagename):
        """Page url for Disqus."""

        if self.config['blog_baseurl']:
            url = urljoin(self.config['blog_baseurl'], pagename)
            if url.endswith('index'):
                url = url[:-5]
            return url


def html_builder_write_doc(self, docname, doctree):
    """Part of :meth:`sphinx.builders.html.StandaloneHTMLBuilder.write_doc`
    method used to convert *doctree* to HTML."""

    destination = StringOutput(encoding='utf-8')
    doctree.settings = self.docsettings

    self.secnumbers = {}
    self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
    self.dlpath = relative_uri(self.get_target_uri(docname), '_downloads')
    self.current_docname = docname
    self.docwriter.write(doctree, destination)
    self.docwriter.assemble_parts()
    return self.docwriter.parts['fragment']


class Post(object):

    """Handle post metadata."""

    def __init__(self, ablog, docname, info):


        self.ablog = ablog
        self.docname = docname
        self.section = info['section']
        self.order = info['order']
        self.date = date = info['date']
        self.update = info['update']
        self.published = date and date < TOMORROW
        self.draft = not self.published
        self.title = info['title']
        self.excerpt = info['excerpt']
        self.doctree = info['doctree']
        self._next = self._prev = -1
        self._computed_date = date or FUTURE

        #self.language = info.get('language')

        # archives
        self.blog = []
        if self.published:
            self.tags = info.get('tags')
            self.author = info.get('author')
            self.category = info.get('category')
            self.location = info.get('location')
            self.language = info.get('language')

            if not self.author and ablog.blog_default_author:
                self.author = ablog.blog_default_author
            if not self.location and ablog.blog_default_location:
                self.location = ablog.blog_default_location
            if not self.language and ablog.blog_default_language:
                self.language = ablog.blog_default_language

            self.archive = self.ablog.archive[self.date.year]
            self.archive.add(self)

        else:
            self.tags = info.get('tags')
            self.author = info.get('author')
            self.category = info.get('category')
            self.location = info.get('location')
            self.language = info.get('language')
            self.archive = []

        self.redirect = info.get('redirect')

        self.options = info

    def __str__(self):
        return self.title

    def __lt__(self, other):
        return (self._computed_date, self.title) < (other._computed_date, other.title)

    def to_html(self, pagename, fulltext=False, drop_h1=True):
        """Return excerpt or *fulltext* as HTML after resolving references
        with respect to *pagename*. By default, first `<h1>` tag is dropped
        from the output. More than one can be dropped by setting *drop_h1*
        to the desired number of tags to be dropped."""

        if fulltext:
            doctree = nodes.document({}, dummy_reporter)
            deepcopy = self.doctree.deepcopy()
            if isinstance(deepcopy, nodes.document):
                doctree.extend(deepcopy.children)
            else:
                doctree.append(deepcopy)
        else:
            doctree = nodes.document({}, dummy_reporter)
            for node in self.excerpt:
                doctree.append(node.deepcopy())
        app = self.ablog.app

        revise_pending_xrefs(doctree, pagename)
        app.env.resolve_references(doctree, pagename, app.builder)

        add_permalinks, app.builder.add_permalinks = (
            app.builder.add_permalinks, False)

        html = html_builder_write_doc(app.builder, pagename, doctree)

        app.builder.add_permalinks = add_permalinks

        if drop_h1:
            html = re.sub('<h1>(.*?)</h1>', '', html, count=abs(int(drop_h1)))
        return html

    @property
    def next(self):
        """Next published post in chronological order."""

        if self._next == -1:
            self.ablog.link_posts()
        return self._next

    @next.setter
    def next(self, post):
        """Set next published post in chronological order."""

        self._next = post

    @property
    def prev(self):
        """Previous published post in chronological order."""

        if self._prev == -1:
            self.ablog.link_posts()
        return self._prev

    @prev.setter
    def prev(self, post):
        """Set previous published post in chronological order."""

        self._prev = post


class Catalog(object):

    """Handles collections of posts."""

    def __init__(self, blog, name, xref, path, reverse=False):

        self.blog = blog
        self.name = name
        self.xref = xref # for creating labels, e.g. `tag-python`
        self.collections = {}

        if path:
            self.path = self.docname = os_path_join(blog.blog_path, path)
        else:
            self.path = self.docname = blog.blog_path

        self._coll_lens = None
        self._min_max = None
        self._reverse = reverse

    def __getitem__(self, name):

        try:
            return self.collections[name]
        except KeyError:
            return self.collections.setdefault(name, Collection(self, name))

    def __setitem__(self, name, item):

        self.collections[name] = item

    def __len__(self):

        return sum(len(coll) for coll in self)

    def __nonzero__(self):

        return len(self) > 0

    def __iter__(self):

        keys = list(self.collections)
        keys.sort(reverse=self._reverse)
        for key in keys:
            yield self.collections[key]

    def add(self, post):
        """Add post to appropriate collection(s) and replace collections
        labels with collection objects."""

        colls = []
        for label in getattr(post, self.name, []):
            coll = self[label]
            if post.published:
                coll.add(post)
            colls.append(coll)
        setattr(post, self.name, colls)

    def _minmax(self):
        """Return minimum and maximum sizes of collections."""

        if (self._coll_lens is None or
            len(self._coll_lens) != len(self.collections)):
            self._coll_lens = [len(coll) for coll in self.collections.values()
                          if len(coll)]
            self._min_max = min(self._coll_lens), max(self._coll_lens)
        return self._min_max


class Collection(object):

    """Posts sharing a label, i.e. tag, category, author, or location."""

    def __init__(self, catalog, label, name=None, href=None, path=None, page=0):

        self.catalog = catalog
        self.label = label
        self.name = name or self.label
        self.href = href
        self.page = page
        self._posts = {}
        self._posts_iter = None
        self._path = path
        self.xref = self.catalog.xref + '-' + slugify(label)
        self._slug = None
        self._html = None

        self.catalog.blog.references[self.xref] = (self.docname, self.name)

    def __str__(self):

        return text_type(self.name)

    def __len__(self):

        return len(self._posts)

    def __nonzero__(self):

        return len(self) > 0

    def __unicode__(self):

        return text_type(self.name)

    def __iter__(self):

        if self._posts_iter is None:
            posts = list(self._posts.values())
            posts.sort(reverse=True)
            self._posts_iter = posts

        for post in self._posts_iter:
            yield post

    def __getitem__(self, key):

        return self._posts.get(key)

    def __contains__(self, item):

        return item in self._posts

    def add(self, post):
        """Add post to the collection."""

        post_name = post.docname
        if post.section:
            post_name += '#' + post.section
        self._posts[post_name] = post

    def relsize(self, maxsize=5, minsize=1):
        """Relative size used in tag clouds."""

        min_, max_ = self.catalog._minmax()

        diff = maxsize - minsize
        if len(self.catalog) == 1 or min_ == max_:
            return int(round(diff / 2. + minsize))

        size = int(1. * (len(self) - min_) / (max_ - min_) * diff + minsize)
        return size

    @property
    def docname(self):
        """Collection page document name."""

        if self._path is None:
            self._path = os_path_join(self.catalog.path, slugify(self.name))
        return self._path

    path = docname



# -*- coding: utf-8 -*-
"""post and postlist directives."""

import os
import sys
from string import Formatter
from datetime import datetime
try:
    from dateutil.parser import parse as date_parser
except ImportError:
    date_parser = None

from docutils import nodes
from sphinx.locale import _
from sphinx.util.nodes import set_source_info
from sphinx.util.compat import Directive, make_admonition
from docutils.parsers.rst import directives
from docutils.utils import relative_path

import ablog
from .blog import Blog, slugify, os_path_join, revise_pending_xrefs

if sys.version_info >= (3, 0):
    text_type = str
else:
    text_type = unicode


class PostNode(nodes.Element):
    """Represent ``post`` directive content and options in document tree."""

    pass


class PostList(nodes.General, nodes.Element):
    """Represent ``postlist`` directive converted to a list of links."""

    pass


class UpdateNode(nodes.Admonition, nodes.Element):
    """Represent ``update`` directive."""

    pass


class PostDirective(Directive):
    """Handle ``post`` directives."""

    _split = lambda a: [s.strip() for s in (a or '').split(',') if s.strip()]

    has_content = True
    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = {
        'tags': _split,
        'author': _split,
        'category': _split,
        'location': _split,
        'language': _split,
        'redirect': _split,
        'title': lambda a: a.strip(),
        'image': int,
        'excerpt': int,
        'exclude': directives.flag,
    }

    def run(self):

        node = PostNode()
        node.document = self.state.document
        set_source_info(self, node)
        self.state.nested_parse(self.content, self.content_offset,
                                node, match_titles=1)

        node['date'] = self.arguments[0] if self.arguments else None
        node['tags'] = self.options.get('tags', [])
        node['author'] = self.options.get('author', [])
        node['category'] = self.options.get('category', [])
        node['location'] = self.options.get('location', [])
        node['language'] = self.options.get('language', [])
        node['redirect'] = self.options.get('redirect', [])
        node['title'] = self.options.get('title', None)
        node['image'] = self.options.get('image', None)
        node['excerpt'] = self.options.get('excerpt', None)
        node['exclude'] = 'exclude' in self.options
        return [node]


class UpdateDirective(Directive):

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):

        ad = make_admonition(UpdateNode, self.name, [_('Updated on')],
                             self.options,
                             self.content, self.lineno, self.content_offset,
                             self.block_text, self.state, self.state_machine)
        ad[0]['date'] = self.arguments[0] if self.arguments else ''
        set_source_info(self, ad[0])
        return ad


class PostListDirective(Directive):
    """Handle ``postlist`` directives."""

    _split = lambda a: set(s.strip() for s in a.split(','))
    has_content = False
    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = False
    option_spec = {
        'tags': _split,
        'author': _split,
        'category': _split,
        'location': _split,
        'language': _split,
        'format': lambda a: a.strip(),
        'date': lambda a: a.strip(),
        'sort': directives.flag,
        'excerpts': directives.flag,
        'list-style': lambda a: a.strip(),
    }

    def run(self):

        node = PostList()
        node.document = self.state.document
        set_source_info(self, node)
        self.state.nested_parse(self.content, self.content_offset,
                                node, match_titles=1)

        node['length'] = int(self.arguments[0]) if self.arguments else None
        node['tags'] = self.options.get('tags', [])
        node['author'] = self.options.get('author', [])
        node['category'] = self.options.get('category', [])
        node['location'] = self.options.get('location', [])
        node['language'] = self.options.get('language', [])
        node['format'] = self.options.get('format', '{date} - {title}')
        node['date'] = self.options.get('date', None)
        node['sort'] = 'sort' in self.options
        node['excerpts'] = 'excerpts' in self.options
        node['image'] = 'image' in self.options
        node['list-style'] = self.options.get('list-style', 'none')
        return [node]


def purge_posts(app, env, docname):
    """Remove post and reference to it from the standard domain when its
    document is removed or changed."""

    if hasattr(env, 'ablog_posts'):
        env.ablog_posts.pop(docname, None)

    filename = os.path.split(docname)[1]
    env.domains['std'].data['labels'].pop(filename, None)


def _get_section_title(section):
    """Return section title as text."""

    for title in section.traverse(nodes.title):
        break
    # A problem with the following is that title may contain pending
    # references, e.g. :ref:`tag-tips`
    return title.astext()


def _get_update_dates(section, docname, post_date_format):
    """Return list of dates of updates found section."""


    update_nodes = list(section.traverse(UpdateNode))
    update_dates = []
    for update_node in update_nodes:
        try:
            update = datetime.strptime(update_node['date'], post_date_format)
        except ValueError:
            raise ValueError('invalid post update date (%s) in %s. '
                             'Expected format: %s' %
                             (update_node['date'], docname, post_date_format))
        substitute = nodes.title(u'',
                                 update_node[0][0].astext() + u' ' +
                                 update.strftime(post_date_format))
        update_node[0].replace_self(substitute)
        # for now, let updates look like note
        update_node['classes'] = ['note', 'update']

        update_dates.append(update)
    return update_dates


def process_posts(app, doctree):
    """Process posts and map posted document names to post details in the
    environment."""

    env = app.builder.env
    if not hasattr(env, 'ablog_posts'):
        env.ablog_posts = {}

    post_nodes = list(doctree.traverse(PostNode))
    if not post_nodes:
        return
    post_date_format = app.config['post_date_format']
    docname = env.docname

    # mark the post as 'orphan' so that
    #   "document isn't included in any toctree" warning is not issued
    app.env.metadata[docname]['orphan'] = True

    blog = Blog(app)
    auto_excerpt = blog.post_auto_excerpt
    multi_post = len(post_nodes) > 1 or blog.post_always_section

    for order, node in enumerate(post_nodes, start=1):
        if node['excerpt'] is None:
            node['excerpt'] = auto_excerpt

        if multi_post:
            # section title, and first few paragraphs of the section of post
            # are used when there are more than 1 posts
            section = node
            while True:
                if isinstance(section, nodes.section):
                    break
                section = node.parent
        else:
            section = doctree

        # get updates here, in the section that post belongs to
        # Might there be orphan updates?
        update_dates = _get_update_dates(section, docname, post_date_format)

        # Making sure that post has a title because all post titles
        # are needed when resolving post lists in documents
        title = node['title'] or _get_section_title(section)

        # creating a summary here, before references are resolved
        excerpt = []
        if node.children:
            if node['exclude']:
                node.replace_self([])
            else:
                node.replace_self(node.children)
            for child in node.children:
                excerpt.append(child.deepcopy())
        elif node['excerpt']:
            count = 0
            for nod in section.traverse(nodes.paragraph):
                excerpt.append(nod.deepcopy())
                count += 1
                if count >= (node['excerpt'] or 0):
                    break
            node.replace_self([])
        else:
            node.replace_self([])
        nimg = node['image'] or blog.post_auto_image
        if nimg:
            for img, nod in enumerate(section.traverse(nodes.image), start=1):
                if img == nimg:
                    excerpt.append(nod.deepcopy())
                    break
        date = node['date']
        if date:
            try:
                date = datetime.strptime(date, post_date_format)
            except ValueError:
                if date_parser:
                    try:
                        date = date_parser(date)
                    except ValueError:
                        raise ValueError('invalid post date in: ' + docname)
                else:
                    raise ValueError('invalid post date (%s) in ' % (date) +
                                     docname +
                                     ". Expected format: %s" % post_date_format)

        else:
            date = None


        # if docname ends with `index` use folder name to reference the document
        # a potential problem here is that there may be files/folders with the
        #   same name, so issuing a warning when that's the case may be a good idea
        folder, label = os.path.split(docname)
        if label == 'index':
            folder, label = os.path.split(folder)
        if not label:
            label = slugify(title)

        section_name = ''
        if multi_post and section.parent is not doctree:
                section_name = section.attributes['ids'][0]
                label += '-' + section_name
        else:
            # create a reference for the post
            # if it is posting the document
            # ! this does not work for sections
            app.env.domains['std'].data['labels'][label] = (docname, label, title)

        if section.parent is doctree:
            section_copy = section[0].deepcopy()
        else:
            section_copy = section.deepcopy()

        # multiple posting may result having post nodes
        for nn in section_copy.traverse(PostNode):
            if nn['exclude']:
                nn.replace_self([])
            else:
                nn.replace_self(node.children)


        postinfo = {
            'docname': docname,
            'section': section_name,
            'order': order,
            'date': date,
            'update': max(update_dates + [date]),
            'title': title,
            'excerpt': excerpt,
            'tags': node['tags'],
            'author': node['author'],
            'category': node['category'],
            'location': node['location'],
            'language': node['language'],
            'redirect': node['redirect'],
            'image': node['image'],
            'exclude': node['exclude'],
            'doctree': section_copy
        }

        if docname not in env.ablog_posts:
            env.ablog_posts[docname] = []
        env.ablog_posts[docname].append(postinfo)

        # instantiate catalogs and collections here
        #  so that references are created and no warnings are issued
        if app.builder.format == 'html':
            stdlabel = env.domains['std'].data['labels']
        else:
            stdlabel = env.intersphinx_inventory.setdefault('std:label', {})
            baseurl = getattr(env.config, 'blog_baseurl').rstrip('/') + '/'
            project, version = env.config.project, text_type(env.config.version)

        for key in ['tags', 'author', 'category', 'location', 'language']:
            catalog = blog.catalogs[key]
            for label in postinfo[key]:
                coll = catalog[label]

        if postinfo['date']:
            coll = blog.archive[postinfo['date'].year]


def process_postlist(app, doctree, docname):
    """Replace `PostList` nodes with lists of posts. Also, register all posts
    if they have not been registered yet."""

    blog = Blog(app)
    if not blog:
        register_posts(app)

    for node in doctree.traverse(PostList):
        colls = []
        for cat in ['tags', 'author', 'category', 'location', 'language']:
            for coll in node[cat]:
                if coll in blog.catalogs[cat].collections:
                    colls.append(blog.catalogs[cat].collections[coll])

        if colls:
            posts = set(blog.posts)
            for coll in colls:
                posts = posts & set(coll)
            posts = list(posts)
            posts.sort(reverse=True)
            posts = posts[:node.attributes['length']]
        else:
            posts = list(blog.recent(node.attributes['length'], docname,
                                          **node.attributes))

        if node.attributes['sort']:
            posts.sort() # in reverse chronological order, so no reverse=True

        fmts = list(Formatter().parse(node.attributes['format']))
        for text, key, __, __ in fmts:
            if key not in {'date', 'title', 'author', 'location', 'language',
                'category', 'tags', None}:
                raise KeyError('{} is not recognized in postlist format'
                    .format(key))

        excerpts = node.attributes['excerpts']
        date_format = node.attributes['date'] or _(blog.post_date_format_short)
        bl = nodes.bullet_list()
        bl.attributes['classes'].append('postlist-style-' + node['list-style'])
        bl.attributes['classes'].append('postlist')
        for post in posts:
            bli = nodes.list_item()
            bl.append(bli)
            par = nodes.paragraph()
            bli.append(par)


            for text, key, __, __ in fmts:
                if text:
                    par.append(nodes.Text(text))
                if key is None:
                    continue
                if key == 'date':
                    par.append(nodes.Text(post.date.strftime(date_format)))
                else:
                    if key == 'title':
                        items = [post]
                    else:
                        items = getattr(post, key)

                    for i, item in enumerate(items, start=1):
                        if key == 'title':
                            ref = nodes.reference()
                            ref['refuri'] = app.builder.get_relative_uri(docname, item.docname)
                            ref['ids'] = []
                            ref['backrefs'] = []
                            ref['dupnames'] = []
                            ref['classes'] = []
                            ref['names'] = []
                            ref['internal'] = True
                            ref.append(nodes.Text(text_type(item)))
                        else:
                            ref = _missing_reference(app, item.xref, docname)
                        par.append(ref)
                        if i < len(items):
                            par.append(nodes.Text(', '))
            if excerpts and post.excerpt:
                for enode in post.excerpt:
                    enode = enode.deepcopy()
                    revise_pending_xrefs(enode, docname)
                    app.env.resolve_references(enode, docname, app.builder)
                    enode.parent = bli.parent
                    bli.append(enode)

        node.replace_self(bl)


def missing_reference(app, env, node, contnode):

    target = node['reftarget']
    return _missing_reference(app, target, node['refdoc'],
                              contnode, node['refexplicit'])

def _missing_reference(app, target, refdoc, contnode=None, refexplicit=False):

    blog = Blog(app)
    if target in blog.references:
        docname, dispname = blog.references[target]

        if 'html' in app.builder.name:
            internal = True
            uri = app.builder.get_relative_uri(refdoc, docname)
        else:
            internal = False
            uri = blog.blog_baseurl + '/' + docname


        newnode = nodes.reference('', '', internal=internal, refuri=uri,
                                  reftitle=dispname)
        if refexplicit:
            newnode.append(contnode)
        else:
            emp = nodes.emphasis()
            newnode.append(emp)
            emp.append(nodes.Text(text_type(dispname)))

        return newnode

def generate_archive_pages(app):
    """Generate archive pages for all posts, categories, tags, authors, and
    drafts."""

    if not ablog.builder_support(app):
        return

    blog = Blog(app)
    for post in blog.posts:
        for redirect in post.redirect:
            yield (redirect, {'redirect': post.docname, 'post': post},
                   'redirect.html')

    atom_feed = bool(blog.blog_baseurl)
    feed_archives = blog.blog_feed_archives
    blog_path = blog.blog_path
    for title, header, catalog  in [
        (_('Authors'), _('Posts by'), blog.author),
        (_('Locations'), _('Posts from'), blog.location),
        (_('Languages'), _('Posts in'), blog.language),
        (_('Categories'), _('Posts in'), blog.category),
        (_('All posts'), _('Posted in'), blog.archive),
        (_('Tags'), _('Posts tagged'), blog.tags),]:

        if not catalog:
            continue

        context = {
            'parents': [],
            'title': title,
            'header': header,
            'catalog': catalog,
            'summary': True,
            'atom_feed': atom_feed,
            'feed_path': blog_path,
        }
        yield (catalog.docname, context, 'catalog.html')

        for collection in catalog:

            if not collection:
                continue
            context = {
                'parents': [],
                'title': u'{} {}'.format(header, collection),
                'header': header,
                'collection': collection,
                'summary': True,
                'atom_feed': atom_feed,
                'feed_path': collection.path if feed_archives else blog_path,
                'archive_feed': atom_feed and feed_archives
            }
            yield (collection.docname, context, 'collection.html')


    #ppp = 5
    #for page, i in enumerate(range(0, len(blog.posts), ppp)):
    if 1:
        context = {
            'parents': [],
            'title': _('All Posts'),
            'header': _('All'),
            'collection': blog.posts,
            'summary': True,
            'atom_feed': atom_feed,
            'feed_path': blog.blog_path,
        }
        docname = blog.posts.docname
        #if page:
        #    docname += '/' + str(page)
        yield (docname, context, 'collection.html')


    context = {
        'parents': [],
        'title': _('Drafts'),
        'collection': blog.drafts,
        'summary': True,
    }
    yield (blog.drafts.docname, context, 'collection.html')


def generate_atom_feeds(app):
    """Generate archive pages for all posts, categories, tags, authors, and
    drafts."""

    if not ablog.builder_support(app):
        return

    blog = Blog(app)


    url = blog.blog_baseurl
    if not url:
        raise StopIteration

    try:
        from werkzeug.contrib.atom import AtomFeed
    except ImportError:
        app.warn("werkzeug is not found, continue without atom feeds support.")
        return

    feed_path = os.path.join(app.builder.outdir, blog.blog_path, 'atom.xml')

    feeds = [(blog.posts,
             blog.blog_path,
             feed_path,
             blog.blog_title,
             os_path_join(url, blog.blog_path, 'atom.xml'))]

    if blog.blog_feed_archives:

        for header, catalog in [
            (_('Posts by'), blog.author),
            (_('Posts from'), blog.location),
            (_('Posts in'), blog.language),
            (_('Posts in'), blog.category),
            (_('Posted in'), blog.archive),
            (_('Posts tagged'), blog.tags),]:

            for coll in catalog:
                # skip collections containing only drafts
                if not len(coll):
                    continue
                folder = os.path.join(app.builder.outdir, coll.path)
                if not os.path.isdir(folder):
                    os.makedirs(folder)

                feeds.append((coll,
                              coll.path,
                              os.path.join(folder, 'atom.xml'),
                              blog.blog_title + u' - ' + header +
                                                u' ' + text_type(coll),
                              os_path_join(url, coll.path, 'atom.xml')))

    # Config options
    feed_length = blog.blog_feed_length
    feed_fulltext = blog.blog_feed_fulltext

    for feed_posts, pagename, feed_path, feed_title, feed_url in feeds:

        feed = AtomFeed(feed_title,
                        title_type='text',
                        url=url,
                        feed_url=feed_url,
                        subtitle=blog.blog_feed_subtitle,
                        generator=('ABlog', 'http://ablog.readthedocs.org',
                                   ablog.__version__))
        for i, post in enumerate(feed_posts):
            if feed_length and i == feed_length:
                break
            post_url = os_path_join(
                url, app.builder.get_target_uri(post.docname))
            if post.section:
                post_url += '#' + post.section

            if blog.blog_feed_titles:
                content = None
            else:
                content = post.to_html(pagename, fulltext=feed_fulltext)
            feed.add(post.title,
                     content=content,
                     title_type='text',
                     content_type='html',
                     author=', '.join(a.name for a in post.author),
                     url=post_url,
                     id=post_url,
                     updated=post.update, published=post.date)

        with open(feed_path, 'w') as out:
            feed_str = feed.to_string()
            try:
                out.write(feed_str.encode('utf-8'))
            except TypeError:
                out.write(feed_str)

    if 0:
        # this is to make the function a generator
        # and make work for Sphinx 'html-collect-pages'
        yield


def register_posts(app):
    """Register posts found in the Sphinx build environment."""

    blog = Blog(app)
    for docname, posts in getattr(app.env, 'ablog_posts', {}).items():
        for postinfo in posts:
            blog.register(docname, postinfo)



import sys
from setuptools import setup

__version__ = ''
with open('ablog/__init__.py') as inp:
    for line in inp:
        if (line.startswith('__version__')):
            exec(line.strip())
            break
long_description = '''
ABlog for Sphinx
================

A Sphinx extension that converts any documentation or personal website project
into a full-fledged blog. See http://ablog.readthedocs.org for details.

.. image:: https://secure.travis-ci.org/abakan/ablog.png?branch=devel
   :target: http://travis-ci.org/#!/abakan/ablog

.. image:: https://readthedocs.org/projects/ablog/badge/?version=latest
   :target: http://ablog.readthedocs.org/
'''

setup(
    name='ablog',
    version=__version__,
    author='Ahmet Bakan',
    author_email='lordnapi@gmail.com',
    description='ABlog for blogging with Sphinx',
    long_description=long_description,
    url='http://ablog.readthedocs.org/',
    packages=['ablog'],
    package_dir={'ablog': 'ablog'},
    package_data={'ablog': [
        'templates/*.html',
        'locale/sphinx.pot',
        'locale/*/LC_MESSAGES/sphinx.*o']},
    license='MIT License',
    keywords=('Sphinx, extension, blogging, atom feeds'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Documentation',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        ],
    provides=['ablog ({0:s})'.format(__version__)],
    install_requires=['Werkzeug', 'Sphinx', 'alabaster', 'invoke', 'python-dateutil'],
    message_extractors={
        'ablog': [
            ('**.html', 'jinja2', None),
            ('**.py',   'python', None),
        ]
    },
    entry_points = {
        'console_scripts': [
            'ablog = ablog.commands:ablog_main',
            'ablog{} = ablog.commands:ablog_main'.format(sys.version_info[0]),
            ],
        },
)
